#
# spec file for package elilo
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%ifarch x86_64
%if 0%{?suse_version} < 1100
BuildRequires:  binutils219
#!BuildIgnore: binutils
%endif
%endif
BuildRequires:  gnu-efi >= 3.0u
BuildRequires:  xz
BuildRequires:  perl(Pod::Man)

Name:           elilo
Summary:        EFI Linux Loader
License:        GPL-2.0-or-later
Group:          System/Boot
Version:        3.16
Release:        10.2
ExclusiveArch:  ia64 %ix86 x86_64
PreReq:         /usr/bin/perl
PreReq:         perl(Getopt::Long)
PreReq:         perl(Pod::Usage)
# "perl" must be in place *before* any package's 'pre' or 'post' section
# can (directly or indirectly) run '/sbin/elilo'!  (bnc#842183)
%ifarch ia64
PreReq:         perl(File::Compare)
%endif
URL:            http://elilo.sourceforge.net/
#ource:         http://downloads.sourceforge.net/elilo/elilo-3.16-all.tar.gz
Source:         elilo-%{version}-source.tar.xz
Source1:        elilo.pl
Source2:        debian.eliloalt.man8
Source3:        elilo.conf.man5
Source4:        http://downloads.sourceforge.net/elilo/%{version}-release-notes.txt
Source9:        elilo-rpmlintrc
Patch0:         elilo-ipv6.diff
Patch1:         elilo-max-conf.diff
Patch2:         elilo-mac-conf.diff
Patch3:         elilo-auto-add_efi_memmap.diff
Patch4:         elilo-blocksize.diff
Patch5:         elilo-text-mode.diff
Patch6:         elilo-textmenu-disable-print-devices.diff
Patch7:         elilo-high_base_mem.diff
Patch10:        elilo-de-debianify.diff
Patch11:        eliloalt-no-date.diff
Patch12:        elilo-binutils-2.36-fix.patch
Patch13:        elilo-c99.patch

BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
The EFI Linux boot loader.

%prep
%autosetup -p1 -n %{name}-%{version}-source

# work around b0rked 'Str'-ops in newer 'gnu-efi' ... :-(
find . -type f -name '*.[ch]' -print0 | xargs -0rn 1 \
  perl -pi -e 's{Str(Chr|n(X?Cpy|Cat))}{eliloStr$1}g'
# facilitate UsrMerge (bsc#1191059) below
%define sbin /usr/sbin

%build
perl -pi -e 's{/usr/lib}{%{_libdir}}' Make.defaults
##################################################################
## DO NOT ADD RPM OPT FLAGS! THIS DOES NOT BUILD AGAINST GLIBC. ##
##################################################################
OPTFLAGS="-fmessage-length=0"
%ifnarch ia64
OPTFLAGS="$OPTFLAGS -mno-red-zone"
%endif
make OPTIMFLAGS="$OPTFLAGS"
perl -pe 's{\@EDITION\@}{%{version}};
	  s{\@LIBDIR\@}{%{_libdir}};
	  s{\@ARCH\@}{%{_target_cpu}};
	  s{/s(bin/get_kernel)}{/usr/$1};
	  s{/sbin/(elilo)}{%{sbin}/$1};
	' < %{SOURCE1} > elilo.pl &&
chmod 555 elilo.pl && touch -r %{SOURCE1} elilo.pl
! grep -F '%%{version}' elilo.pl
pod2man -s 8 -c "System Boot" -r "SuSE Linux" \
    -n elilo -d "%{version}" elilo.pl elilo.8
touch -r elilo.pl elilo.8

%install
install -d $RPM_BUILD_ROOT%{_libdir}/efi $RPM_BUILD_ROOT%{sbin}
install -p -m 444 elilo.efi $RPM_BUILD_ROOT%{_libdir}/efi
install tools/eliloalt $RPM_BUILD_ROOT%{sbin}
install -p -m 555 elilo.pl $RPM_BUILD_ROOT%{sbin}/elilo
install -D -p -m 644 elilo.8 $RPM_BUILD_ROOT/usr/share/man/man8/elilo.8
install -D -p -m 644 %{SOURCE2} $RPM_BUILD_ROOT/usr/share/man/man8/eliloalt.8
install -D -p -m 644 %{SOURCE3} $RPM_BUILD_ROOT/usr/share/man/man5/elilo.conf.5
install -p -m 644 %{SOURCE4} RELEASE-NOTES
diff -q docs/README.txt docs/elilo.txt && rm -f docs/README.txt

%post
if [ -r /etc/sysconfig/bootloader ]; then
  . /etc/sysconfig/bootloader
  if [ "$LOADER_TYPE" = "elilo" -a -r /etc/elilo.conf ]; then
    %{sbin}/elilo -v || :
  fi
fi

%files
%defattr(-, root, root)
%doc README README.* TODO docs/*.txt RELEASE-NOTES
%{_libdir}/efi
%{sbin}/elilo
%{sbin}/eliloalt
/usr/share/man/man5/*
/usr/share/man/man8/*

%changelog
* Thu Jul 25 2024 Filip Kastl <filip.kastl@suse.com>
- Add elilo-c99.patch so that the package conforms to the C99
  standard and builds on i586 with GCC 14.
* Thu Feb 22 2024 Dominique Leuenberger <dimstar@opensuse.org>
- Use %%autosetup macro. Allows to eliminate the usage of deprecated
  %%patchN
* Tue Aug 16 2022 rw@suse.com
- elilo.spec, elilo.pl
  * Move to '/usr/sbin'.  (bsc#1191059)
* Wed Feb  3 2021 Martin Liška <mliska@suse.cz>
- Add elilo-binutils-2.36-fix.patch in order to fix build
  with binutils 2.36.
* Fri Nov 23 2018 rw@suse.com
- elilo.efi
  * Try to properly allocate high_base_mem.  (bsc#1000769)
    (elilo-high_base_mem.diff)
* Thu Nov 22 2018 rw@suse.com
- elilo.spec
  * Work around glitches introduced by gnu-efi.
  * Add '-mno-red-zone' to work around Microsoft/SystemV AMD64 ABI
    discrepancies.  (bsc#953502)
- elilo.pl
  * Support 'ucode=' for XEN.  (bsc#1102567)
  * SecureBoot: Support detached configuration template.
  * Add support for 'UUID='/'LABEL=' to specify EFI system partition
    and fix bug introduced by NVMe device handling.  (bsc#917195)
  * Handle NVMe device names.  (fate#317591)
  * Don't abort, when "skip" is announced.  (bsc#917130)
- elilo.efi
  * Remove special handling for '?' in textmenu-mode.  (bsc#928546)
    (elilo-textmenu-disable-print-devices.diff)
* Sat Mar  3 2018 bwiedemann@suse.com
- Do not include rebuild counter in rpm to make build-compare happy
* Thu Jul 31 2014 dimstar@opensuse.org
- Rename rpmlintrc to %%{name}-rpmlintrc.
  Follow the packaging guidelines.
* Fri Oct 25 2013 rw@suse.de
- elilo.efi
  * Update to elilo-3.16 to fix OBS download check.
    Essentially "white-space" changes, plus bumping version number,
    minus Debian idiosyncrasies.
- elilo.spec
  * Remove date string from 'eliloalt'.
  * Avoid duplication of 'elilo.txt'.
- elilo.pl
  * Support for 'UUID=' and 'LABEL=' to specify root file-system.
* Thu Oct 24 2013 rw@suse.de
- elilo.spec
  * Add perl modules to 'PreReq'.  (bnc#842183)
- Update openSUSE to elilo-3.14 from SLE11.
* Fri Aug 16 2013 rw@suse.de
- elilo.pl
  * SecureBoot: cope with separate '/boot' file-system.  (bnc#825932)
  * SecureBoot: improve detection of file-system UUIDs.  (bnc#828835)
  * Correctly handle installation to 'BOOT'.
* Wed May 29 2013 rw@suse.com
- elilo.pl
  * Always delete old EBM entries to fix ordering.  (bnc#819900)
  * Prevent XEN 'default' in 'elilo.conf'.  (bnc#819900c5)
  * Add man-page section about '--refresh-EBM' failure. (bnc#821109)
* Wed May  8 2013 rw@suse.com
- elilo.pl
  * SecureBoot: create only one Boot Manager entry.  (bnc#818961)
* Fri Apr 19 2013 rw@suse.com
- elilo.efi
  * Clear console on startup.  (bnc#812799)
  * Avoid crash caused by EFI memory map changes.  (bnc#800035)
- elilo.pl
  * Work around chainloading issue with XEN.  (bnc#812109)
* Wed Mar 27 2013 rw@suse.com
- elilo.pl
  * Fix handling of missing 'default' again.
* Wed Mar 27 2013 rw@suse.com
- elilo.pl
  * Implement chainloading of XEN for SecureBoot. (bnc#809702)
  * Maintain 'default' in SecureBoot config. (bnc#811608c28)
  * Handle failure of 'efibootmgr'.  (bnc#809702c10)
  * Improve verbose messages to be less alarming.
* Fri Mar  8 2013 rw@suse.com
- elilo.efi
  * Force default configs at the end of the network search list.
    (bnc#789137)
- elilo.pl
  * Disable use of 'shim'-loader options.  (bnc#798043)
  * Fix solitary 'secure-boot' for real.
* Tue Feb 12 2013 rw@suse.com
- elilo.pl
  * Revert work-around for 'shim'-loader option parsing. (bnc#798043)
  * Treat 'secure-boot' identical to 'secure-boot = on'.
  * Fix 'SecureBoot' sysfs variable reading.
* Fri Feb  8 2013 rw@suse.com
- elilo.efi
  * Now really introduce 'block-size'. (bnc#681242)
  * Clarify warning message on GOP failure.
- elilo.pl
  * Synchronize check for "Secure Boot" with YaST.
* Fri Jan 18 2013 rw@suse.com
- elilo.efi
  * Make automatic appending of 'add_efi_memmap' to kernel command
    line default, and introduce config-file option (add-efi-memmap)
    to control this (auto|false).  (fate#314210, bnc#772245)
  * Bring network search list more in line with 'PXELINUX'. (bnc#789137)
  * Introduce 'block-size' as global option in 'elilo.conf'. (bnc#681242)
- elilo.pl
  * Work around limitations in 'shim'-loader option parsing. (bnc#798043)
  * Don't use temporary file to establish load options.
* Wed Dec 19 2012 rw@suse.com
- elilo.pl
  * Add support UEFI Secure Boot (via 'grub.efi'). (fate#314485)
  * Don't try to write 'xen.cfg', if no valid XEN section is found
    and treat "VMM" EFI-binaries as "mandatory".  (bnc#792100)
  * Support more than 10 EFI Boot Manager (EBM) entries.
  * Stop using '--write-signature' for 'efibootmgr' (as non-GPT
    disk-labels aren't supported anyway).
  * Sort creation of EBM entries according to default boot section.
  * Fix initial handling of obsolete files.
* Mon Jan 16 2012 rw@suse.de
- elilo.pl
  * Cope with misguided '/dev/mapper/' entries in 'fstab'.  (bnc#717828)
* Fri Nov 25 2011 rw@suse.de
- elilo.pl
  * Allow empty 'vmmopts' for 'xen.efi'.  (bnc#731288)
  * Implement refreshing of EFI Boot Manager entries.  (bnc#717828)
* Mon Sep 19 2011 rw@suse.de
- elilo.pl
  * Fix handling of 'xen.cfg'.  (bnc#706110)
  * Record installed files and remove only those in the future.
* Sun Sep 18 2011 jengelh@medozas.de
- elilo.spec
  * Remove redundant/obsolete tags/sections from specfile
    (cf. packaging guidelines)
* Mon Aug  1 2011 rw@suse.de
- elilo.efi
  * Allow booting in an IPv6 network environment.  (fate#311995)
- elilo.pl
  * Add support for 'xen.efi'.  (fate#311376)
* Fri Jun 17 2011 rw@suse.de
- elilo.efi
  * Update to elilo-3.14.  (fate#311532)
- elilo.spec
  * Follow openSUSE migration from libexecdir to libdir.
- elilo.pl
  * Don't initialize '$[' to 0.
  * Adapt to replacement of libexecdir with libdir.
* Fri Dec 17 2010 coolo@novell.com
- Update openSUSE to elilo-3.12 from SLE11 to fix EFI boot (bnc#659368)
  (see ChangeLog for a detailed log)
* Mon Apr 12 2010 rw@suse.de
- elilo.efi
  * Handle EFI memory maps larger than E820_MAX.  (bnc#594516)
* Tue Mar 30 2010 rw@suse.de
- elilo.efi
  * Update boot protocol for 2.6.30.  (bnc#560443)
  * Allocate initrd below kernel specified limit.  (bnc#577570)
  * Load kernel relocated.  (bnc#447843, 516215, 568848, 587858)
- elilo.pl
  * Add handling of VMM for XEN.  (bnc#578926)
* Tue Jan 26 2010 rw@suse.de
- elilo.efi
  * PRELIMINARY debugging patch for x86_64.  (bnc#560443)
* Fri Nov 20 2009 rw@suse.de
- elilo.efi
  * Update to elilo-3.12.
* Tue Sep 22 2009 rw@suse.de
- elilo.spec
  * Submit to SLE11.  (bnc#494042)
* Mon Jul 20 2009 coolo@novell.com
- elilo.spec
  * replace libexecdir with libdir
* Thu May 28 2009 ro@suse.de
- elilo.spec
  * Use binutils219 on SLE10 SP3.
* Fri May 15 2009 rw@suse.de
- elilo.efi
  * Update to elilo-3.10.  (fate#306032)
* Thu Apr 30 2009 ro@suse.de
- elilo.spec
  * Submit to SLE10 SP3.  (fate#306032)
* Mon Feb  9 2009 rw@suse.de
- elilo.efi
  * Allow booting x86_64 from network.  (bnc#473914)
  * Introduce new per-image option 'text-mode' for x86_64, which can
    be used to sidestep "dubious" GOP implementations.  (bnc#466570)
  * Minor cleanup.
* Tue Dec  9 2008 rw@suse.de
- elilo.efi
  * Second attempt to work around firmware problem.  (bnc#437486)
- elilo.conf
  * Provide stub man page.  (bnc#435648)
* Fri Nov 21 2008 rw@suse.de
- elilo.efi
  * Significant code cleanup from CVS.  (bnc#443565)
  * First attempt to work around firmware problem.  (bnc#437486)
- elilo.pl
  * Improve removal of obsolete files.  (bnc#398416)
- eliloalt
  * Support sysfs interface.  (bnc#440489)
  * Provide man page.  (bnc#440197)
* Mon Oct 27 2008 rw@suse.de
- elilo.pl
  * Mask 'relocatable' (unless supported).  (bnc#438276)
  * Remove old files from target directory.  (bnc#398416)
* Fri Oct 17 2008 olh@suse.de
- elilo.spec
  * Add ExclusiveArch ia64 x86 x86_64
* Thu Jul 31 2008 rw@suse.de
- elilo.efi
  * Update to final elilo-3.8 for uEFI support.  (fate#301882)
- elilo.pl
  * Preserve time-stamps but not permissions.  (bnc#394331)
  * Unconditionally strip deprecated 'read-only' from 'elilo.conf'.
  * Support '$LIBEXECDIR' to please uEFI/x86_64 and
    relocate efi-binaries from 'elilo' to 'efi'.  (fate#301882)
* Thu Feb 28 2008 ro@suse.de
- elilo.spec
  * No strip for elilo.efi during install (unknown binary format).
* Sat May 26 2007 schwab@suse.de
- elilo.spec
  * Revert last change.
* Sat May 26 2007 ro@suse.de
- elilo.spec
  * Added gnu-efi-devel to buildreq.
* Wed May  9 2007 rw@suse.de
- elilo.efi
  * Reduce non-"command line" strings back to 512.  (#256676)
  * Fix another buffer overrun.  (#256676)
* Fri Feb 23 2007 schwab@suse.de
- elilo.spec
  * Disable stack protector.
* Tue Feb  6 2007 bwalle@suse.de
- elilo.efi
  * Increase command line size to 2048.  (#242702)
* Fri Nov 10 2006 schwab@suse.de
- elilo.efi
  * Fix spelling.
* Thu Feb  2 2006 rw@suse.de
- elilo.efi
  * Update to final 3.6
    (Fixed vmcode_name initialization in textmenu chooser)
* Wed Jan 25 2006 mls@suse.de
- elilo.spec
  * Converted neededforbuild to BuildRequires.
* Fri Jan 20 2006 schwab@suse.de
- elilo.spec
  * Don't strip binaries.
* Mon Jan  9 2006 rw@suse.de
- elilo.efi
  * Fix vmcode_name initialization in textmenu chooser.  (#140519)
* Fri Dec  9 2005 rw@suse.de
- elilo.pl
  * Fix formatting of man-page.
* Wed Dec  7 2005 rw@suse.de
- elilo.efi
  * Update to 3.5-pre2.
- elilo.pl
  * Make "best effort" instead of giving up early.
* Sat Aug 13 2005 schwab@suse.de
- elilo.pl
  * Check that the default label is defined.
* Fri Aug  5 2005 schwab@suse.de
- elilo.pl
  * Barf on duplicate labels.
* Wed Apr 13 2005 schwab@suse.de
- elilo.efi
  * Fix for new assembler.
* Fri Feb 18 2005 schwab@suse.de
- elilo.efi
  * Clear extra memory in initrd block so that initramfs works.
* Wed Oct  6 2004 rw@suse.de
- elilo.efi
  * Fix simple-chooser (#45493)
* Thu May 27 2004 rw@suse.de
- elilo.pl
  * Make all image-objects (kernel/initrd) *optional* by default (#41315)
* Mon Feb 23 2004 rw@suse.de
- elilo.efi
  * Use '$RPM_OPT_FLAGS' with '-fno-strict-aliasing' appended.
- elilo.pl
  * Take care of FPSWA.
  * Try to avoid copying the same files more than once.
* Mon Nov 24 2003 rw@suse.de
- Provide '/sbin/elilo' to simplify separation of '/boot' and the
  EFI FAT-partition (bug #21644)
- Move 'elilo.efi' to '/usr/lib/elilo'
* Mon Sep  1 2003 schwab@suse.de
- Update to final 3.4 to remove extraneous debug output.
* Tue Aug 26 2003 rw@suse.de
- Add patch from Greg Edwards (SGI)   (#29340)
* Fri Aug 22 2003 schwab@suse.de
- Update to elilo 3.4.
  * Improved netbooting.
* Tue Jun 10 2003 schwab@suse.de
- Add fix for netbooting.
* Tue Apr  8 2003 schwab@suse.de
- Add two fixes from author.
* Thu Oct 17 2002 schwab@suse.de
- Update to elilo 3.3a.
  * More memory management bugs fixed.
* Fri Sep 13 2002 schwab@suse.de
- Fix %%post.
- Fix eliloalt.
* Wed Sep 11 2002 schwab@suse.de
- Fix buffer overrun.
* Mon Sep  2 2002 schwab@suse.de
- Move elilo.conf to /boot/efi/SuSE.
* Tue Aug 27 2002 schwab@suse.de
- Update to elilo 3.3.
* Thu May 16 2002 schwab@suse.de
- Move boot loader to /boot/efi/SuSE.
* Tue Mar  5 2002 schwab@suse.de
- Update to elilo 3.2.
- Make elilo.conf and startup.nsh %%ghost files.
* Sat Aug 18 2001 schwab@suse.de
- Update to elilo 3.1.
* Wed Jul 25 2001 schwab@suse.de
- Add two patches from author.
* Wed Jul 18 2001 schwab@suse.de
- Split from gnu-efi packages.
- New version 3.0.
